/**
 * Copyright (c) 2010-2016, Abel Hegedus, IncQuery Labs Ltd.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * Contributors:
 * Abel Hegedus - initial API and implementation
 */
package org.eclipse.viatra.query.tooling.ui.queryresult.handlers;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import java.util.List;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.IWorkbenchSite;
import org.eclipse.ui.dialogs.ElementTreeSelectionDialog;
import org.eclipse.ui.dialogs.ISelectionStatusValidator;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.viatra.query.runtime.api.AdvancedViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngineManager;
import org.eclipse.viatra.query.tooling.ui.browser.ViatraQueryToolingBrowserPlugin;
import org.eclipse.viatra.query.tooling.ui.queryresult.QueryResultView;
import org.eclipse.viatra.query.tooling.ui.queryresult.util.ViatraQueryEngineContentProvider;
import org.eclipse.viatra.query.tooling.ui.queryresult.util.ViatraQueryEngineLabelProvider;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * @author Abel Hegedus
 */
@SuppressWarnings("all")
public class OpenManagedEngineHandler extends AbstractHandler {
  @Override
  public Object execute(final ExecutionEvent event) throws ExecutionException {
    IWorkbenchSite _activeSite = HandlerUtil.getActiveSite(event);
    IWorkbenchPage _page = _activeSite.getPage();
    final IViewPart resultView = _page.findView(QueryResultView.ID);
    if ((resultView instanceof QueryResultView)) {
      QueryResultView queryResultView = ((QueryResultView) resultView);
      IWorkbenchPartSite _site = queryResultView.getSite();
      Shell _shell = _site.getShell();
      AdvancedViatraQueryEngine _openDialog = this.openDialog(_shell);
      final AdvancedViatraQueryEngine engine = ((AdvancedViatraQueryEngine) _openDialog);
      boolean _notEquals = (!Objects.equal(engine, null));
      if (_notEquals) {
        queryResultView.loadExistingEngine(engine);
      }
    }
    return null;
  }
  
  private AdvancedViatraQueryEngine openDialog(final Shell shell) {
    ViatraQueryEngineContentProvider _viatraQueryEngineContentProvider = new ViatraQueryEngineContentProvider();
    final Procedure1<ViatraQueryEngineContentProvider> _function = new Procedure1<ViatraQueryEngineContentProvider>() {
      @Override
      public void apply(final ViatraQueryEngineContentProvider it) {
        it.setTraverseResources(true);
        it.setTraverseEObjects(true);
      }
    };
    final ViatraQueryEngineContentProvider contentProvider = ObjectExtensions.<ViatraQueryEngineContentProvider>operator_doubleArrow(_viatraQueryEngineContentProvider, _function);
    ViatraQueryEngineLabelProvider _viatraQueryEngineLabelProvider = new ViatraQueryEngineLabelProvider();
    final ElementTreeSelectionDialog engineSelectionDialog = new ElementTreeSelectionDialog(shell, _viatraQueryEngineLabelProvider, contentProvider);
    engineSelectionDialog.setTitle("Managed engine selection");
    engineSelectionDialog.setMessage("Select a managed engine (root elements of the tree)!");
    ViatraQueryEngineManager _instance = ViatraQueryEngineManager.getInstance();
    engineSelectionDialog.setInput(_instance);
    engineSelectionDialog.setHelpAvailable(false);
    engineSelectionDialog.setValidator(new ISelectionStatusValidator() {
      @Override
      public IStatus validate(final Object[] selection) {
        boolean _and = false;
        int _size = ((List<Object>)Conversions.doWrapArray(selection)).size();
        boolean _equals = (_size == 1);
        if (!_equals) {
          _and = false;
        } else {
          Iterable<AdvancedViatraQueryEngine> _filter = Iterables.<AdvancedViatraQueryEngine>filter(((Iterable<?>)Conversions.doWrapArray(selection)), AdvancedViatraQueryEngine.class);
          int _size_1 = IterableExtensions.size(_filter);
          boolean _equals_1 = (_size_1 == 1);
          _and = _equals_1;
        }
        if (_and) {
          return new Status(IStatus.OK, ViatraQueryToolingBrowserPlugin.PLUGIN_ID, 0, "", null);
        } else {
          return new Status(IStatus.ERROR, ViatraQueryToolingBrowserPlugin.PLUGIN_ID, 0, "", null);
        }
      }
    });
    engineSelectionDialog.open();
    final Object[] result = engineSelectionDialog.getResult();
    boolean _notEquals = (!Objects.equal(result, null));
    if (_notEquals) {
      Iterable<AdvancedViatraQueryEngine> _filter = Iterables.<AdvancedViatraQueryEngine>filter(((Iterable<?>)Conversions.doWrapArray(result)), AdvancedViatraQueryEngine.class);
      return IterableExtensions.<AdvancedViatraQueryEngine>head(_filter);
    }
    return null;
  }
}
