/**
 * 
 *   Copyright (c) 2010-2015, Csaba Debreceni, Istvan Rath and Daniel Varro
 *   All rights reserved. This program and the accompanying materials
 *   are made available under the terms of the Eclipse Public License v1.0
 *   which accompanies this distribution, and is available at
 *   http://www.eclipse.org/legal/epl-v10.html
 *  
 *   Contributors:
 *     Csaba Debreceni - initial API and implementation
 *  
 */
package org.eclipse.viatra.transformation.views.traceability.patterns.internal;

import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedPatternGroup;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.transformation.views.traceability.patterns.internal.TraceTargetsQuerySpecification;
import org.eclipse.viatra.transformation.views.traceability.patterns.util.EmptyTraceQuerySpecification;
import org.eclipse.viatra.transformation.views.traceability.patterns.util.Param2targetQuerySpecification;
import org.eclipse.viatra.transformation.views.traceability.patterns.util.Trace2objectQuerySpecification;
import org.eclipse.viatra.transformation.views.traceability.patterns.util.Trace2paramQuerySpecification;
import org.eclipse.viatra.transformation.views.traceability.patterns.util.Trace2targetQuerySpecification;
import org.eclipse.viatra.transformation.views.traceability.patterns.util.TraceQuerySpecification;

/**
 * A pattern group formed of all patterns defined in helpers.vql.
 * 
 * <p>A private group that includes private patterns as well. Only intended use case is for pattern testing.
 * 
 * <p> From package org.eclipse.viatra.transformation.views.traceability.patterns, the group contains the definition of the following patterns: <ul>
 * <li>emptyTrace</li>
 * <li>traceTargets</li>
 * <li>trace2target</li>
 * <li>trace2param</li>
 * <li>trace2object</li>
 * <li>param2target</li>
 * <li>trace</li>
 * </ul>
 * 
 * @see IPatternGroup
 * 
 */
@SuppressWarnings("all")
public final class HelpersAll extends BaseGeneratedPatternGroup {
  /**
   * Access the pattern group.
   * 
   * @return the singleton instance of the group
   * @throws ViatraQueryException if there was an error loading the generated code of pattern specifications
   * 
   */
  public static HelpersAll instance() throws ViatraQueryException {
    if (INSTANCE == null) {
        INSTANCE = new HelpersAll();
    }
    return INSTANCE;
  }
  
  private static HelpersAll INSTANCE;
  
  private HelpersAll() throws ViatraQueryException {
    querySpecifications.add(EmptyTraceQuerySpecification.instance());
    querySpecifications.add(TraceTargetsQuerySpecification.instance());
    querySpecifications.add(Trace2targetQuerySpecification.instance());
    querySpecifications.add(Trace2paramQuerySpecification.instance());
    querySpecifications.add(Trace2objectQuerySpecification.instance());
    querySpecifications.add(Param2targetQuerySpecification.instance());
    querySpecifications.add(TraceQuerySpecification.instance());
  }
}
